@@ -99,15 +99,18 @@ class StreamMessageBuilder:
         input_order: int,
         last_read_time: float,
         last_write_time: float,
-        last_process_time: float
+        last_process_time: float,
+        cached_frame_id: Optional[str] = None,
     ) -> Dict[str, Any]:
         """Build complete stream message.
 
+        Supports both normal frames (with content) and cached frames (empty content + cached_frame_id).
+
         Args:
-            frame_data: Encoded frame bytes
+            frame_data: Encoded frame bytes (can be empty for cached frames)
             stream_key: Stream identifier
             stream_group_key: Stream group identifier
-            codec: Video codec
+            codec: Video codec (use "cached" for cached frames)
             metadata: Frame metadata
             topic: Topic name
             broker_config: Broker configuration string
@@ -115,12 +118,14 @@ class StreamMessageBuilder:
             last_read_time: Last read time
             last_write_time: Last write time
             last_process_time: Last process time
+            cached_frame_id: Frame ID to use cached results from (None for normal frames)
 
         Returns:
             Complete message dictionary
         """
         # Store content as raw bytes (NO base64 encoding for performance)
         # Redis/Kafka will handle binary data directly
+        # For cached frames, frame_data will be empty bytes
         if frame_data and not self.strip_input_content:
             content_data = frame_data  # Raw bytes
         else:
@@ -167,10 +172,17 @@ class StreamMessageBuilder:
                 "last_write_time_sec": last_write_time,
                 "last_process_time_sec": last_process_time,
             },
-            "content": content_data,  # Raw binary data (no base64)
+            "content": content_data,  # Raw binary data (no base64) - empty for cached frames
             "input_hash": self._compute_fast_hash(frame_data) if frame_data else None,
         }
 
+        # Add cached_frame_id if this is a cached frame
+        if cached_frame_id:
+            input_stream["cached_frame_id"] = cached_frame_id
+            # Include similarity score for debugging/metrics
+            if "similarity_score" in metadata:
+                input_stream["similarity_score"] = metadata["similarity_score"]
+
         # Add passthrough metadata
         passthrough_keys = {
             "similarity_score", "skip_reason",
