@@ -32,6 +32,7 @@ class StreamStatistics:
         self.process_times_history: Dict[str, List[float]] = {}
         self.frame_sizes_history: Dict[str, List[int]] = {}
         self.frame_timestamps_history: Dict[str, List[float]] = {}
+        self.encoding_times_history: Dict[str, List[float]] = {}  # NEW: encoding time tracking
 
         # Per-stream input order tracking
         self.input_order: Dict[str, int] = {}
@@ -60,7 +61,8 @@ class StreamStatistics:
         read_time: float,
         write_time: float,
         process_time: float,
-        frame_size: Optional[int] = None
+        frame_size: Optional[int] = None,
+        encoding_time: float = 0.0
     ):
         """Update timing statistics for a stream.
 
@@ -70,6 +72,7 @@ class StreamStatistics:
             write_time: Time spent writing/sending frame
             process_time: Total processing time
             frame_size: Size of encoded frame in bytes (ACG frame size)
+            encoding_time: Time spent encoding frame (NEW)
         """
         key = self._normalize_key(stream_key)
         timestamp = time.time()
@@ -88,11 +91,13 @@ class StreamStatistics:
             self.process_times_history[key] = []
             self.frame_sizes_history[key] = []
             self.frame_timestamps_history[key] = []
+            self.encoding_times_history[key] = []
 
         self.read_times_history[key].append(read_time)
         self.write_times_history[key].append(write_time)
         self.process_times_history[key].append(process_time)
         self.frame_timestamps_history[key].append(timestamp)
+        self.encoding_times_history[key].append(encoding_time)
 
         if frame_size is not None:
             self.frame_sizes_history[key].append(frame_size)
@@ -104,6 +109,7 @@ class StreamStatistics:
             self.write_times_history[key] = self.write_times_history[key][-self.MAX_HISTORY_SIZE:]
             self.process_times_history[key] = self.process_times_history[key][-self.MAX_HISTORY_SIZE:]
             self.frame_timestamps_history[key] = self.frame_timestamps_history[key][-self.MAX_HISTORY_SIZE:]
+            self.encoding_times_history[key] = self.encoding_times_history[key][-self.MAX_HISTORY_SIZE:]
             if len(self.frame_sizes_history[key]) > self.MAX_HISTORY_SIZE:
                 self.frame_sizes_history[key] = self.frame_sizes_history[key][-self.MAX_HISTORY_SIZE:]
     
@@ -154,7 +160,7 @@ class StreamStatistics:
         write_time: float
     ):
         """Log periodic statistics.
-        
+
         Args:
             stream_key: Stream identifier
             read_time: Time spent reading frame
@@ -169,7 +175,82 @@ class StreamStatistics:
                 f"Timing: read={read_time*1000:.1f}ms, encode={encoding_time*1000:.1f}ms, "
                 f"write={write_time*1000:.1f}ms"
             )
-    
+
+    def log_detailed_stats(self, stream_key: str) -> None:
+        """Log comprehensive metrics for a stream.
+
+        Args:
+            stream_key: Stream identifier
+        """
+        stats = self.get_timing_statistics(stream_key)
+        if not stats:
+            return
+
+        # Calculate additional metrics
+        total_frames = self.frames_sent + self.frames_skipped + self.frames_diff_sent
+        skip_rate = (self.frames_skipped / total_frames * 100) if total_frames > 0 else 0
+
+        # FPS metrics
+        fps_stats = stats.get("fps", {})
+        fps_current = fps_stats.get("avg", 0)
+        fps_min = fps_stats.get("min", 0)
+        fps_max = fps_stats.get("max", 0)
+
+        # Latency breakdown (ms)
+        read_ms = stats.get("read_time_ms", {}).get("avg", 0)
+        encoding_ms = stats.get("encoding_time_ms", {}).get("avg", 0)
+        write_ms = stats.get("write_time_ms", {}).get("avg", 0)
+        process_ms = stats.get("process_time_ms", {}).get("avg", 0)
+
+        # Frame size stats (KB)
+        frame_size_stats = stats.get("frame_size_bytes", {})
+        frame_size_avg_kb = frame_size_stats.get("avg", 0) / 1024
+        frame_size_min_kb = frame_size_stats.get("min", 0) / 1024
+        frame_size_max_kb = frame_size_stats.get("max", 0) / 1024
+
+        # Throughput (KB/s)
+        throughput_kbps = (frame_size_avg_kb * fps_current) if fps_current > 0 else 0
+
+        self.logger.info(
+            f"Stream Metrics [{stream_key}]: "
+            f"FPS={fps_current:.1f} (min={fps_min:.1f}, max={fps_max:.1f}) | "
+            f"Latency: read={read_ms:.1f}ms, encode={encoding_ms:.1f}ms, write={write_ms:.1f}ms, total={process_ms:.1f}ms | "
+            f"Frames: sent={self.frames_sent}, skipped={self.frames_skipped} ({skip_rate:.1f}%) | "
+            f"Frame size: {frame_size_avg_kb:.1f}KB (min={frame_size_min_kb:.1f}, max={frame_size_max_kb:.1f}) | "
+            f"Throughput: {throughput_kbps:.1f} KB/s"
+        )
+
+    def log_aggregated_stats(self) -> None:
+        """Log aggregated metrics across all streams."""
+        total_frames_sent = self.frames_sent
+        total_frames_skipped = self.frames_skipped
+        total_frames_diff = self.frames_diff_sent
+        total_frames = total_frames_sent + total_frames_skipped + total_frames_diff
+
+        if total_frames == 0:
+            return
+
+        skip_rate = (total_frames_skipped / total_frames * 100)
+        diff_rate = (total_frames_diff / total_frames * 100)
+
+        # Aggregate FPS across all streams
+        all_fps = []
+        for stream_key in self.last_read_times.keys():
+            stats = self.get_timing_statistics(stream_key)
+            if stats and "fps" in stats:
+                fps_avg = stats["fps"].get("avg", 0)
+                if fps_avg > 0:
+                    all_fps.append(fps_avg)
+
+        avg_fps = sum(all_fps) / len(all_fps) if all_fps else 0
+
+        self.logger.info(
+            f"Gateway Aggregate Metrics: "
+            f"Total frames: {total_frames} (sent={total_frames_sent}, skipped={total_frames_skipped}, diff={total_frames_diff}) | "
+            f"Skip rate: {skip_rate:.1f}%, Diff rate: {diff_rate:.1f}% | "
+            f"Avg FPS across {len(all_fps)} streams: {avg_fps:.1f}"
+        )
+
     def get_transmission_stats(self, video_codec: str, active_streams: int) -> Dict[str, Any]:
         """Get comprehensive transmission statistics.
         
@@ -243,6 +324,7 @@ class StreamStatistics:
         result = {
             "read_time_ms": {"min": 0, "max": 0, "avg": 0, "count": 0},
             "write_time_ms": {"min": 0, "max": 0, "avg": 0, "count": 0},
+            "encoding_time_ms": {"min": 0, "max": 0, "avg": 0, "count": 0},  # NEW
             "process_time_ms": {"min": 0, "max": 0, "avg": 0, "count": 0},
             "frame_size_bytes": {"min": 0, "max": 0, "avg": 0, "count": 0},
             "fps": {"min": 0, "max": 0, "avg": 0},
@@ -278,6 +360,16 @@ class StreamStatistics:
                 "count": len(process_times),
             }
 
+        # Calculate encoding time statistics
+        if key in self.encoding_times_history and self.encoding_times_history[key]:
+            encoding_times = self.encoding_times_history[key]
+            result["encoding_time_ms"] = {
+                "min": min(encoding_times) * 1000,
+                "max": max(encoding_times) * 1000,
+                "avg": (sum(encoding_times) / len(encoding_times)) * 1000,
+                "count": len(encoding_times),
+            }
+
         # Calculate frame size statistics
         if key in self.frame_sizes_history and self.frame_sizes_history[key]:
             frame_sizes = self.frame_sizes_history[key]
@@ -330,6 +422,7 @@ class StreamStatistics:
             self.process_times_history.clear()
             self.frame_sizes_history.clear()
             self.frame_timestamps_history.clear()
+            self.encoding_times_history.clear()
             self.logger.debug("Cleared timing history for all streams")
         else:
             # Clear specific stream
@@ -344,6 +437,8 @@ class StreamStatistics:
                 self.frame_sizes_history[key].clear()
             if key in self.frame_timestamps_history:
                 self.frame_timestamps_history[key].clear()
+            if key in self.encoding_times_history:
+                self.encoding_times_history[key].clear()
             self.logger.debug(f"Cleared timing history for stream: {stream_key}")
     
     def reset(self):
