@@ -5,7 +5,9 @@ import cv2
 import requests
 import os
 import tempfile
+import hashlib
 from pathlib import Path
+from urllib.parse import urlparse, urlunparse
 from typing import Union, Optional, Tuple, Dict, Any
 
 
@@ -21,11 +23,19 @@ class VideoSourceConfig:
 
 
 class VideoCaptureManager:
-    """Manages video capture from various sources with retry logic and caching."""
+    """Manages video capture from various sources with retry logic and caching.
+    
+    Features URL deduplication: if multiple cameras use the same video URL
+    (ignoring query parameters like AWS signed URL tokens), the video is only
+    downloaded once and the local path is shared between cameras.
+    """
     
     def __init__(self):
         """Initialize video capture manager."""
+        # Maps full URL -> local file path (for backwards compatibility)
         self.downloaded_files: Dict[str, str] = {}
+        # Maps normalized URL (without query params) -> local file path (for deduplication)
+        self._normalized_url_to_path: Dict[str, str] = {}
         self.temp_dir = Path(tempfile.gettempdir()) / "matrice_streaming_cache"
         self.temp_dir.mkdir(exist_ok=True)
         self.logger = logging.getLogger(__name__)
@@ -123,14 +133,20 @@ class VideoCaptureManager:
     
     def cleanup(self):
         """Clean up downloaded temporary files."""
-        for filepath in self.downloaded_files.values():
+        # Collect unique file paths (since multiple URLs may point to the same file)
+        unique_files = set(self.downloaded_files.values())
+        unique_files.update(self._normalized_url_to_path.values())
+        
+        for filepath in unique_files:
             try:
                 if os.path.exists(filepath):
                     os.remove(filepath)
                     self.logger.debug(f"Removed temp file: {filepath}")
             except Exception as e:
                 self.logger.warning(f"Failed to remove temp file {filepath}: {e}")
+        
         self.downloaded_files.clear()
+        self._normalized_url_to_path.clear()
     
     # Private methods
     
@@ -138,33 +154,113 @@ class VideoCaptureManager:
         """Check if source is a downloadable URL (not RTSP)."""
         return (source.startswith('http://') or source.startswith('https://')) and not source.startswith('rtsp')
     
+    def _normalize_url(self, url: str) -> str:
+        """Normalize URL by stripping query parameters.
+        
+        This allows URLs that point to the same file but have different
+        query parameters (e.g., AWS signed URLs with different tokens)
+        to be recognized as the same resource.
+        
+        Args:
+            url: Full URL with potential query parameters
+            
+        Returns:
+            Normalized URL without query parameters
+        """
+        parsed = urlparse(url)
+        # Rebuild URL without query string and fragment
+        normalized = urlunparse((
+            parsed.scheme,
+            parsed.netloc,
+            parsed.path,
+            '',  # params
+            '',  # query
+            ''   # fragment
+        ))
+        return normalized
+    
+    def _get_url_hash(self, normalized_url: str) -> str:
+        """Generate a short hash for the normalized URL.
+        
+        This is used for creating unique but consistent file names.
+        
+        Args:
+            normalized_url: URL without query parameters
+            
+        Returns:
+            Short hash string
+        """
+        return hashlib.md5(normalized_url.encode()).hexdigest()[:12]
+    
     def _download_video_file(self, url: str, stream_key: str) -> Optional[str]:
-        """Download video file from URL and cache it locally."""
+        """Download video file from URL and cache it locally.
+        
+        Uses URL deduplication: if the same video (by normalized URL without 
+        query parameters) has already been downloaded, returns the existing 
+        local path instead of downloading again.
+        
+        Args:
+            url: Video file URL (may include query parameters like AWS signatures)
+            stream_key: Stream identifier
+            
+        Returns:
+            Local file path or None if download failed
+        """
         try:
-            # Check cache
+            # Normalize URL to check for duplicate downloads
+            # (same file but different query params, e.g., different AWS signatures)
+            normalized_url = self._normalize_url(url)
+            
+            # Generate a consistent filename using URL hash
+            file_ext = Path(url.split('?')[0]).suffix or '.mp4'
+            url_hash = self._get_url_hash(normalized_url)
+            expected_path = self.temp_dir / f"video_{url_hash}{file_ext}"
+            expected_path_str = str(expected_path)
+            
+            # Quick check: if file already exists on disk, use it
+            if os.path.exists(expected_path):
+                self.logger.info(
+                    f"Reusing existing video file for {stream_key}: {expected_path} "
+                    f"(already downloaded)"
+                )
+                # Update caches
+                self.downloaded_files[url] = expected_path_str
+                self._normalized_url_to_path[normalized_url] = expected_path_str
+                return expected_path_str
+            
+            # Check memory cache for exact URL
             if url in self.downloaded_files:
                 local_path = self.downloaded_files[url]
                 if os.path.exists(local_path):
-                    self.logger.info(f"Using cached video file: {local_path}")
+                    self.logger.info(f"Using cached video file (exact URL match): {local_path}")
                     return local_path
             
-            # Download
+            # Check memory cache for normalized URL
+            if normalized_url in self._normalized_url_to_path:
+                local_path = self._normalized_url_to_path[normalized_url]
+                if os.path.exists(local_path):
+                    self.logger.info(
+                        f"Reusing previously downloaded file for {stream_key}: {local_path} "
+                        f"(same base URL, different query params)"
+                    )
+                    self.downloaded_files[url] = local_path
+                    return local_path
+            
+            # Download the file
             self.logger.info(f"Downloading video file from: {url}")
             response = requests.get(url, stream=True, timeout=VideoSourceConfig.DOWNLOAD_TIMEOUT)
             response.raise_for_status()
             
-            # Save to temp directory
-            file_ext = Path(url.split('?')[0]).suffix or '.mp4'
-            local_path = self.temp_dir / f"{stream_key}_{int(time.time())}{file_ext}"
-            
-            with open(local_path, 'wb') as f:
+            with open(expected_path, 'wb') as f:
                 for chunk in response.iter_content(chunk_size=VideoSourceConfig.DOWNLOAD_CHUNK_SIZE):
                     f.write(chunk)
             
-            # Cache
-            self.downloaded_files[url] = str(local_path)
-            self.logger.info(f"Downloaded video file to: {local_path}")
-            return str(local_path)
+            # Cache for both exact URL and normalized URL
+            self.downloaded_files[url] = expected_path_str
+            self._normalized_url_to_path[normalized_url] = expected_path_str
+            
+            self.logger.info(f"Downloaded video file to: {expected_path}")
+            return expected_path_str
             
         except Exception as e:
             self.logger.error(f"Failed to download video file: {e}")
