@@ -15,6 +15,7 @@ from matrice_common.rpc import RPC
 from .streaming_gateway import StreamingGateway
 from .streaming_gateway_utils import StreamingGatewayUtil
 from .metrics_reporter import MetricsManager, MetricsConfig
+from .streaming_status_listener import StreamingStatusListener
 
 
 class StreamingAction:
@@ -59,7 +60,7 @@ class StreamingAction:
         monitoring_interval: float = 30.0,  # Heartbeat sent every monitoring_interval seconds
         auto_restart: bool = True,
         max_restart_attempts: int = 3,
-        action_id_check_interval: float = 60.0,
+        action_id_check_interval: float = 600.0,  # 10 minutes (was 60 seconds)
         enable_event_listening: bool = True,
     ):
         """Initialize StreamingAction.
@@ -144,7 +145,7 @@ class StreamingAction:
 
         # Initialize utility and gateway objects
         self.gateway_util = StreamingGatewayUtil(
-            session, self.streaming_gateway_id, self.server_id
+            session, self.streaming_gateway_id, self.server_id, action_id=self.action_id
         )
         self.streaming_gateway: Optional[StreamingGateway] = None
 
@@ -174,6 +175,9 @@ class StreamingAction:
         # Initialize metrics manager
         self.metrics_manager: Optional[MetricsManager] = None
 
+        # Initialize status listener for stop commands
+        self.status_listener: Optional[StreamingStatusListener] = None
+
         logging.info(
             "StreamingAction initialized successfully for gateway: %s",
             self.streaming_gateway_id,
@@ -259,6 +263,7 @@ class StreamingAction:
                 video_codec=self.video_codec,  # Pass video codec setting
                 force_restart=True,  # Always force restart for orchestrator
                 enable_event_listening=self.enable_event_listening,  # Enable dynamic event updates
+                action_id=self.action_id,  # Pass action_id for API requests
             )
 
             # Start streaming
@@ -296,8 +301,26 @@ class StreamingAction:
             # Start health monitoring thread
             self._start_monitoring()
 
-            # Update status to running
-            # self.update_status("STG_RUNNING", "SUCCESS", "Streaming gateway started successfully")
+            # Initialize and start status listener for stop commands
+            try:
+                self.status_listener = StreamingStatusListener(
+                    session=self.session,
+                    streaming_gateway_id=self.streaming_gateway_id,
+                    action_id=self.action_id,
+                    on_stop_callback=self.stop
+                )
+                self.status_listener.start()
+                logging.info("Status listener started for stop commands")
+            except Exception as exc:
+                logging.warning(f"Failed to initialize status listener: {exc}", exc_info=True)
+                self.status_listener = None
+
+            # Update gateway status to running
+            try:
+                self.gateway_util.update_status("running")
+                logging.info("Gateway status updated to 'running'")
+            except Exception as exc:
+                logging.warning(f"Failed to update gateway status to running: {exc}")
 
             logging.info("StreamingAction started successfully")
 
@@ -350,6 +373,14 @@ class StreamingAction:
                 if self._monitor_thread.is_alive():
                     logging.warning("Monitor thread did not stop gracefully")
 
+            # Stop status listener
+            if self.status_listener:
+                try:
+                    logging.info("Stopping status listener...")
+                    self.status_listener.stop()
+                except Exception as exc:
+                    logging.warning(f"Error stopping status listener: {exc}")
+
             # Stop metrics manager
             if self.metrics_manager:
                 try:
@@ -467,8 +498,13 @@ class StreamingAction:
         """
         Check if the current action ID matches the streaming gateway's actionRecordID.
 
+        Handles transient errors gracefully:
+        - 502 Bad Gateway: Skip check, continue streaming (server temporarily unavailable)
+        - 404 Not Found: Stop streaming (gateway may be deleted)
+        - Other API errors: Skip check, continue streaming (don't stop on transient issues)
+
         Returns:
-            bool: True if action ID matches, False otherwise
+            bool: True if action ID matches or check should be skipped, False if mismatch or gateway deleted
         """
         try:
             self.stats["action_id_checks"] += 1
@@ -476,12 +512,13 @@ class StreamingAction:
             # Fetch current streaming gateway details
             gateway_details = self.gateway_util.get_streaming_gateway_by_id()
 
-            if not gateway_details:
+            # Handle API failures gracefully - don't stop streaming on transient errors
+            if gateway_details is None:
                 logging.warning(
-                    "Failed to fetch streaming gateway details for action ID check"
+                    "Action ID check: Failed to fetch gateway details (API returned None), skipping check"
                 )
-                self.stats["action_id_check_failures"] += 1
-                return False
+                # Don't increment failure counter - this is expected during transient issues
+                return True  # Continue streaming, don't stop on API errors
 
             gateway_action_id = gateway_details.get("actionRecordID", "")
 
@@ -502,9 +539,12 @@ class StreamingAction:
             return True
 
         except Exception as exc:
-            self.stats["action_id_check_failures"] += 1
-            logging.error("Error checking action ID match: %s", str(exc), exc_info=True)
-            return False
+            # Unknown error - don't stop streaming on unexpected issues
+            logging.warning(
+                "Action ID check: Unexpected error '%s', skipping check to avoid false shutdown",
+                str(exc)
+            )
+            return True  # Continue streaming, be conservative
 
     def _start_monitoring(self):
         """Start the health monitoring thread."""
