@@ -3,6 +3,7 @@ import re
 from typing import Optional, Union, Dict, List
 import logging
 import time
+from concurrent.futures import ThreadPoolExecutor, as_completed
 from matrice_common.session import Session
 
 
@@ -27,11 +28,12 @@ class InputStream:
 class StreamingGatewayUtil:
 
     def __init__(
-        self, session: Session, streaming_gateway_id: str, server_id: str = None
+        self, session: Session, streaming_gateway_id: str, server_id: str = None, action_id: str = None
     ):
         self.session = session
         self.streaming_gateway_id = streaming_gateway_id
         self.server_id = server_id
+        self.action_id = action_id
         if not self.server_id and self.streaming_gateway_id:
             self.server_id = self.get_streaming_gateway_by_id().get("serverId")
         
@@ -159,12 +161,24 @@ class StreamingGatewayUtil:
 
     def get_input_streams(self) -> List[InputStream]:
         """Get cameras as list of InputStream objects with proper configuration."""
+        # Time the first 3 API calls
+        start = time.time()
         cameras = self.get_cameras()
+        cameras_time = time.time() - start
+        logging.info(f"API call get_cameras() took {cameras_time:.3f}s")
+
+        start = time.time()
         camera_groups = self.get_camera_groups()
+        groups_time = time.time() - start
+        logging.info(f"API call get_camera_groups() took {groups_time:.3f}s")
+
+        start = time.time()
         input_topics = self.get_streaming_input_topics()
+        topics_time = time.time() - start
+        logging.info(f"API call get_streaming_input_topics() took {topics_time:.3f}s")
 
         if not cameras:
-            logging.warning("No cameras found for streaming gateway", exc_info=True)
+            logging.warning("No cameras found for streaming gateway")
             return []
 
         # Create lookup dictionaries
@@ -173,6 +187,34 @@ class StreamingGatewayUtil:
             topic["cameraId"]: topic["topicName"] for topic in (input_topics or [])
         }
 
+        # Identify FILE cameras that need simulated stream URLs
+        file_cameras = [c for c in cameras if c.get("protocolType") == "FILE"]
+
+        # Fetch simulated URLs concurrently
+        stream_url_lookup = {}
+        if file_cameras:
+            max_workers = min(len(file_cameras), 100)  # Max 100 concurrent requests
+            start = time.time()
+            with ThreadPoolExecutor(max_workers=max_workers) as executor:
+                future_to_camera = {
+                    executor.submit(self.get_simulated_stream_url, c["id"]): c["id"]
+                    for c in file_cameras
+                }
+                for future in as_completed(future_to_camera):
+                    camera_id = future_to_camera[future]
+                    try:
+                        result = future.result()
+                        if result and result.get("url"):
+                            stream_url_lookup[camera_id] = result["url"]
+                    except Exception as e:
+                        logging.warning(f"Failed to get stream URL for camera {camera_id}: {e}")
+
+            urls_time = time.time() - start
+            logging.info(
+                f"Concurrent fetch of {len(file_cameras)} stream URLs took {urls_time:.3f}s "
+                f"(max_workers={max_workers})"
+            )
+
         input_streams = []
 
         for camera in cameras:
@@ -192,10 +234,9 @@ class StreamingGatewayUtil:
             simulate_video = False
 
             if camera.get("protocolType") == "FILE":
-                # Get simulated stream URL for file type cameras
-                stream_url_data = self.get_simulated_stream_url(camera_id)
-                if stream_url_data and stream_url_data.get("url"):
-                    source = stream_url_data["url"]
+                # Use pre-fetched URL from concurrent lookup
+                if camera_id in stream_url_lookup:
+                    source = stream_url_lookup[camera_id]
                     simulate_video = True
                 else:
                     # Fallback to simulation video path
@@ -216,7 +257,7 @@ class StreamingGatewayUtil:
                 camera_key=camera.get("cameraName", "Unknown Camera"),
                 camera_group_key=camera_group.get(
                     "cameraGroupName", "Unknown Camera Group"
-                ),  # TODO: change to location not id
+                ),
                 camera_location=camera_group.get(
                     "locationId", "Unknown Camera Location"
                 ),
@@ -227,10 +268,7 @@ class StreamingGatewayUtil:
 
             input_streams.append(input_stream)
 
-        logging.info(
-            f"Created {len(input_streams)} input streams for streaming gateway",
-            exc_info=True,
-        )
+        logging.info(f"Created {len(input_streams)} input streams for streaming gateway")
         return input_streams
 
     def start_streaming(self) -> Optional[Dict]:
@@ -345,9 +383,11 @@ class StreamingGatewayUtil:
 
         def _get_redis_connection_info():
             try:
-                response = self.session.rpc.get(
-                    f"/v1/actions/redis_servers/{server_id}"
-                )
+                # Build URL with actionId query parameter if available
+                url = f"/v1/actions/redis_servers/{server_id}"
+                if self.action_id:
+                    url += f"?actionId={self.action_id}"
+                response = self.session.rpc.get(url)
                 if response.get("success", False):
                     data = response.get("data")
                     if (
@@ -413,9 +453,10 @@ class StreamingGatewayUtil:
                             f"/v1/actions/get_kafka_server/{server_id}"
                         )
                     else:
-                        response = self.session.rpc.get(
-                            f"/v1/actions/redis_servers/{server_id}"
-                        )
+                        url = f"/v1/actions/redis_servers/{server_id}"
+                        if self.action_id:
+                            url += f"?actionId={self.action_id}"
+                        response = self.session.rpc.get(url)
                     logging.error("Last response received: %s", response)
                 except Exception as exc:
                     logging.error(
@@ -465,4 +506,74 @@ class StreamingGatewayUtil:
             return success
         except Exception as e:
             logging.error(f"Failed to send heartbeat: {e}", exc_info=True)
-            return False
\ No newline at end of file
+            return False
+
+
+def input_stream_to_camera_config(input_stream: InputStream) -> Dict:
+    """Convert InputStream dataclass to camera_config dict for WorkerManager.
+
+    This adapter function converts the InputStream configuration format used by
+    StreamingGateway to the dictionary format expected by WorkerManager and
+    AsyncCameraWorker.
+
+    Args:
+        input_stream: InputStream dataclass instance
+
+    Returns:
+        Dict compatible with WorkerManager/AsyncCameraWorker
+    """
+    return {
+        'stream_key': input_stream.camera_key or f"camera_{input_stream.camera_id}",
+        'camera_id': input_stream.camera_id,
+        'source': input_stream.source,
+        'topic': input_stream.camera_input_topic or f"{input_stream.camera_id}_input_topic",
+        'fps': input_stream.fps,
+        'quality': input_stream.quality,
+        'width': input_stream.width,
+        'height': input_stream.height,
+        'camera_location': input_stream.camera_location or "Unknown",
+        'stream_group_key': input_stream.camera_group_key or "default",
+        'simulate_video_file_stream': input_stream.simulate_video_file_stream,
+    }
+
+
+def build_stream_config(
+    gateway_util: StreamingGatewayUtil,
+    server_type: str,
+    service_id: str
+) -> Dict:
+    """Build stream_config dict for WorkerManager from gateway connection info.
+
+    This function retrieves connection information from the API and builds
+    a configuration dictionary suitable for WorkerManager and AsyncCameraWorker.
+
+    Args:
+        gateway_util: StreamingGatewayUtil instance for API calls
+        server_type: 'kafka' or 'redis'
+        service_id: Streaming gateway ID (used as service_id)
+
+    Returns:
+        Dict with connection configuration for WorkerManager
+    """
+    # Get connection info (this already waits for server to be ready)
+    connection_info = gateway_util.get_and_wait_for_connection_info(
+        server_type=server_type
+    )
+
+    # Build stream config
+    stream_config = {
+        **connection_info,
+        'service_id': service_id,
+    }
+
+    # Add Redis-specific batching defaults if applicable
+    # These will be auto-optimized by WorkerManager based on camera count
+    if server_type.lower() == 'redis':
+        stream_config.update({
+            'pool_max_connections': 500,  # High connection pool for 1000+ cameras
+            'enable_batching': True,
+            'batch_size': 10,
+            'batch_timeout': 0.01,
+        })
+
+    return stream_config
\ No newline at end of file
